/* ServerSocketChannelImpl.java -- 
   Copyright (C) 2002 Free Software Foundation, Inc.

This file is part of GNU Classpath.

GNU Classpath is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU Classpath is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU Classpath; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version. */

package gnu.java.nio;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.SocketAddress;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.nio.channels.spi.SelectorProvider;

class ServerSocketChannelImpl extends ServerSocketChannel
{
  ServerSocket sock_object;
  int fd;
//   int local_port;
  boolean blocking = true;
  boolean connected = false;
//   InetSocketAddress sa;

  private static native int SocketAccept (ServerSocketChannelImpl server,
                                          SocketChannelImpl s);

  protected ServerSocketChannelImpl (SelectorProvider provider)
  {
    super (provider);
    fd = SocketChannelImpl.SocketCreate ();

    try
      {
        sock_object = new ServerSocket ();
      }
    catch (IOException e)
      {
        System.err.println ("ServerSocket could not be created.");
      }
  }
 
  public void finalizer()
  {
    if (connected)
	    {
        try
          {
            close();
          }
        catch (Exception e)
          {
          }
	    }
  }

  protected void implCloseSelectableChannel ()
  {
    connected = false;
    SocketChannelImpl.SocketClose (fd);
    fd = SocketChannelImpl.SocketCreate ();
  }

  protected void implConfigureBlocking (boolean block)
  {
    blocking = block;
  }

  public SocketChannel accept ()
  {
    SocketChannelImpl result = new SocketChannelImpl (provider ());
    result.sa = new InetSocketAddress (0);
    int res = SocketAccept (this,result);
    return result;
  }

  public ServerSocket socket ()
  {
    return sock_object;
  }
}
